/*
 * horse_race.c
 * --- CORSA DI CAVALLI ---
 * Il programma simula una corsa di N cavalli.
 * L'utente scommette sul cavallo vincente.
 *
 * SVOLGIMENTO DELLA CORSA
 * Ad ogni turno si calcola la posizione di ciascun cavallo sommando un
 * numero random da 1 a STEP, finche` almeno un cavallo raggiunge il
 * traguardo (GOAL).
 * A questo punto il cavallo che ha percorso piu` strada e` il vincitore.
 *
 * Author: Marco Della Vedova - University of Pavia
 * Update: Mar.21, 2013
 */
#include <stdlib.h>
#include <stdio.h>
#include <time.h>
#include <unistd.h>

#define GOAL 100 /* La distanza del traguardo */
#define STEP 5 /* La massima distanza percorribile in un turno */
#define MAXCAV 99  /* Il massimo numero di cavalli partecipanti */
#define GOALGRAPHIC 70 /* La distanza dell'arrivo sullo schermo */

/* Funzione per pulire lo schermo. Usa un *escape char*. */
void clear()
{
	printf("\033[H\033[2J");
	return;
}


/* Stampa la posizione *p* del cavallo *index*, all'interno del percorso.
 * gli spqazi prima dell'indice del cavallo indicano lo spazio gi percorso,
 * quelli successivi quantificano la distanza rimanente
 * Ad esempio, se p=3, index=5, GOAL=10, verra` stampata la stringa seguente:
 * "  5       |\n"
 */
void print_pos( int p, int index )         /*???????!!!!!!!!????????*/
{
	int i, pos;
	float scale = (float)GOAL / GOALGRAPHIC;
	pos = (int)(p/scale/2); /* diviso due perche` ogni 'i' ha due caratteri*/
	for( i=0; i<GOALGRAPHIC/2; i++ ){
		if( i!=pos ){
			printf("  ");
		}else{
			printf("%02d", index);
		}
	}
	printf("|");
	if( p > GOAL ) printf("%02d", index);
	printf("\n");
}


/* Trova il massimo tra i valori del vettore delle distanze coperte e restituisce il
 * rispettivo indice.
 */
int find_leader( int vett[], int dim  )
{
	int i = 0, max = vett[0], lead = 0;
	for(i=0; i<dim; i++)  /* solo minore perche' il massimo indice del vettore e' dim - 1 */
	{
		if(vett[i] > max)
		{
			max = vett[i];
			lead = i;
		}
	}
	return lead;
}


int main()
{
	char buf[100]; /* per usare la fgets */
	int bet=0; /* il cavallo su cui si scommette */
	int winner=0; /* il cavallo vincitore */
	int c[MAXCAV]={0};  /* vettore delle distanze coperte */
	int i=0;  /* contatore */
	int N=0;  /* numero partecipanti */
    int leader=0; /* Il cavallo al comando della gara */

	/* Richiesta inserimento scommessa */
	printf("Inserisci il numero di cavalli partecipanti [2 to %i]:\n", MAXCAV);
	fgets(buf, sizeof(buf), stdin);
	N = atoi(buf);
	if (N < 2 || N > MAXCAV)
	{
		printf("Valore non accettabile\n");
		return -1;
	}
	printf("Inserisci il numero del cavallo su cui scommetti [1 to %i]:\n", N);
	scanf("%i", &bet);
	if (bet < 1 || bet > N)
	{
		printf("Scommessa non accettabile\n");
		return -1;
	}

	srand( time(NULL) ); /* inizializzazione del generatore di numeri casuali */

	/* Ciclo che incrementa la posizione dei cavalli */
	do{

		/* I cavalli si muovono. rand() restituisce un int casuale */
		/* Stampo a video le varie posizioni */
		clear();
		i = 0;
		while(i<N)
		{
			c[i] += 1 + rand()%STEP;
			print_pos(c[i], i+1);  /* i+1 perche' il vettore e' indicizzato da 0, i cavalli partono da 1 */
			i++;
		}
		printf("\nHai scommesso su...%i\n", bet);        /* ricordo la scelta compiuta */

		/* Controllo se c'e` un vincitore */
		leader = find_leader(c, N);
		if(c[leader] > GOAL)
			winner = leader + 1;     /* perche' il vettore e' indicizzato da 0 */

		usleep(500000); /* Metto in pausa il programma per 500000 microsec
		                 * in modo da poter "vedere" cosa succede */
	}while( winner == 0 );


	/* Stampo le informazioni finali e esco */
	printf("\nIl vincitore e'... %i\n", winner);
	if (winner == bet)
	{
		printf("Hai vinto! Purtroppo l'allibratore e' fuggito con i soldi.\n");
	}
	else
	{
		printf("Hai perso! Sfigatto!\n");
	}
	return 0;
}





